/******************************************************************************
 * Author: Mario Roessler
 * Date:   2008/12/01
 * Description: puts characters from UART to a 4x27 LCD Display
 ******************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "lcd_1.h"
#include "uart.h"

#define F_CPU 16000000UL  // Clock speed = 16MHz
#define XTAL 16000000

#define delay(us)  _delayFourCycles( ( ( 1*(XTAL/4000) )*us)/1000 )

#define DISP_LINE_LENGTH    27
#define UART_BAUD_RATE      19200 

//these dividers need to be defined manually because
//calculation does not fit to the correct speed (rounding)
//values for 20MHz clock:
#define BAUD_38400    25
#define BAUD_57600    16
#define BAUD_115200    8


/*  Pin Definitions:          (Pollin Board)
 *  LCD: PortA  Data  4,5,6,7 (Pin 14,16,18,20)
 *              RS    3       (Pin 12)
 *       PortC  RW    2       (Pin 5)
 *              E0/E1 3,4     (Pin 7,9)
 *  UART select:
 *       PORTA  2/1/0 2,1,0   (Pin 10,8,6)
 */

void uart_autoset(void)
{
/*
       Bit
  Baud 210
  ========
  1200 000
  4800 001
  9600 010
 14400 011
 19200 100
 38400 101
 57600 110
115200 111
*/

   uint8_t speed = 0;
   char buffer[5];

   //speed = ((int)PINA & 0x07);  // read in lowest 3 bits of PORTA
   //speed = PINA;
   //speed = speed + 30;
   speed = (PIND&0x1C)>>2;
   itoa((int)speed, buffer, 5);
   
   if ((int)speed == 0)
      {
	     uart_init( UART_BAUD_SELECT(1200,F_CPU) );
      } else if ((int)speed == 1)
	  {
	     uart_init( UART_BAUD_SELECT(4800,F_CPU) );
      } else if ((int)speed == 2)
	  {
	     uart_init( UART_BAUD_SELECT(9600,F_CPU) );
      } else if ((int)speed == 3)
	  {
	     uart_init( UART_BAUD_SELECT(14400,F_CPU) );
      } else if ((int)speed == 4)
	  {
	     uart_init( UART_BAUD_SELECT(19200,F_CPU) );
      } else if ((int)speed == 5)
	  {
		 uart_init(BAUD_38400);
      } else if ((int)speed == 6)
	  {
	     uart_init(BAUD_57600);
      } else if ((int)speed == 7)
	  {
	     uart_init(BAUD_115200);
      } else
	  {
	     uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );
      }
   
   UCSRB |= RXCIE;

   sei();  // enable Interrupts

   //uart_puts(buffer);
   //uart_puts("\n\r");

}

void hello_world(void)
{
   // Hello
   uart_puts("\n\rUART2LCD v09a\n\r");
   lcd_puts(1,"UART2LCD v09a\n");
   lcd_puts(2,"by Mario Roessler\n");
}

void uart_echo(void)
{
   unsigned char buffer;
   
   buffer = uart_getc();
   if (buffer && UART_NO_DATA)
   {
      uart_putc(buffer);
	  if (((unsigned)buffer == '\r'))
	  {
         uart_puts("\n\r");
		 lcd_putc(1,'\n');
	  }
	  else
	  {
         lcd_putc(1,buffer);
	  }
   }


}


void type_buffered(char* buf0,char* buf1,char* buf2)
{
   char buffer;
   int i=DISP_LINE_LENGTH; // Number of characters each line +1
   int firstcr = 0;
   
   //lcd_clrscr(1);  // clear display
   //lcd_gotoxy(1,1,0);
   

   while(1==1)
   {
   buffer = uart_getc();

   if (buffer && UART_NO_DATA)
   {
      uart_putc(buffer);
	  if (((unsigned)buffer == '\r'))
	  {
         uart_puts("\n\r");
		 if (firstcr==1)
		 {
		    /* wrap lines up...*/
            lcd_clrscr(1);
			lcd_gotoxy(1,0,0);
            lcd_puts(1,buf0);
	        lcd_gotoxy(1,0,1);
	        lcd_puts(1,buf1);
			lcd_clrscr(2);
            lcd_gotoxy(2,0,0);
            lcd_puts(2,buf2);
			/* wrap buffers */
			//pbuf0 = pbuf1;
			//pbuf1 = pbuf2;
			//if(pbuf2 == buf2) pbuf2=buf0;
			//else if(pbuf2 == buf1) pbuf2=buf2;
			//else if(pbuf2 == buf0) pbuf2=buf1;
			strcpy(buf0,buf1);
			strcpy(buf1,buf2);
			strncpy(buf2,"\0",1); // clear buffer
			i=DISP_LINE_LENGTH;
		  } 
		  else 
		  {
		     firstcr=1;  
			 i=DISP_LINE_LENGTH;
          }
	  }
	  else
	  {
         //lcd_putc(1,buffer);
		 //strcat(line,"\n");
		 if (i==DISP_LINE_LENGTH)
		 {
		    /* wrap lines up...*/
            lcd_clrscr(1);
			lcd_gotoxy(1,0,0);
            lcd_puts(1,buf0);
	        lcd_gotoxy(1,0,1);
	        lcd_puts(1,buf1);
			lcd_clrscr(2);
            lcd_gotoxy(2,0,0);
            lcd_puts(2,buf2);
			/* wrap buffers */
			//pbuf0 = pbuf1;
			//pbuf1 = pbuf2;
			//if(pbuf2 == buf2) pbuf2=buf0;
			//else if(pbuf2 == buf1) pbuf2=buf2;
			//else if(pbuf2 == buf0) pbuf2=buf1;
			strcpy(buf0,buf1);
			strcpy(buf1,buf2);
			strncpy(buf2,"\0",1); // clear buffer
			lcd_gotoxy(2,0,1);   
		 }
		 strncat(buf2,&buffer,1); //http://www.mikrocontroller.net/topic/47505
		 lcd_putc(2,buffer);      // put just the last character to display, would be too slow otherwise
		 i--;
		 firstcr=0;
		 if (i==0) return;
	  }
   }
   }

}

void uart_lcd(void)
{
   unsigned char buffer;
   
   buffer = uart_getc();
   if (buffer && UART_NO_DATA)
   {
      lcd_putc(1,buffer);
   }


}

int main (void)
{

   //int act_line=0;
   int loop;

   /* Line Buffer Variables */
   char buf0[28],buf1[28],buf2[28];

   // Initialize UART
   //uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );
   //sei();  // enable Interrupts

   // print/set detected UART speed
   uart_autoset();

   // Initialize Display
   lcd_init(1,LCD_DISP_ON);
   lcd_init(2,LCD_DISP_ON);
   lcd_clrscr(1);
   lcd_clrscr(2);

   hello_world();
   
   /* wait for a while */
   loop=256;
   while(loop)
   {
      delay(100000);
	  delay(100000);
	  delay(100000);
	  loop--;
   }

   /* clear display */
   lcd_clrscr(1);
   lcd_clrscr(2);

   /* clear all buffers */
   strcpy(buf0,"\0");
   strcpy(buf1,"\0");
   strcpy(buf2,"\0");

   while(1==1) 
   {

	  
	  type_buffered(buf0,buf1,buf2);  //write to the lowest line


   }; // just looping around
}
